%% System of ODEs describing dynamics of the system.

% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
%                                                                         %
%         NF-kB regulatory system with autocrine TNFa stimulation         %
%                                                                         %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %

function dy = NFkB_autocrine_modelD(t,y, NA,NI,NT,R)

% Remark: Number of TNFR1 receptors, R, all NF-kB molecules, NFkBtot,
%         and gene numbers, NA,NI,NT, are defined externally in file
%         NFkB_autocrine_simulationD.m and are passed here as arguments 
%         if needed.

% Remark: Default values are provided in comments in square brackets.

%% Cell parameters --------------------------------------------------------

kv = 5;         % [5]  ratio of C:N volumes (inverse of N:C ratio)

KN  =   10^5;   % [  10^5]  total number of IKKK molecules
KNN = 2*10^5;   % [2*10^5]  total number of IKK molecules


% --- TNFR1 activation and signal transduction cascade --------------------

csec =   10^-5; % [10^-5]    TNFa secretion
cdeg = 2*10^-4; % [2*10^-4]  TNFa degradation,
                %                    4*10^-4 => t1/2 = 30min (double deg), 
                %                    Savas: t1/2=60min (normal deg)
cb =     10^4;  % [10^4]     autocrine efficiency
kb = 1.2*10^-5; % [1.2*10^-5] receptor activation rate 
                %                    (about 2/min at 10 ng dose)
kf = 1.2*10^-3; % [1.2*10^-3] receptor inactivation rate,
                %                    6*10^-4 => t1/2 = 20min [Grell 1998]; 
                %                    dissociation t1/2 = 33min; 
                %                    internalization t1/2 = 10-20min 
ka =   10^-5;   % [10^-5]    IKKK activation rate (at most 1/s)
ka20=  10^5;    % [10^5]     A20 TNFR1 block
ki =    0.01;   % [0.01]     IKKK inactivation rate
k1 = 6*10^-10;  % [6*10^-10] IKKn activation by IKKK_a (at most 1/s)
k2 =   10^4;    % [10^4]     IKKa inactivation by A20
k3 =    0.002;  % [0.002]    IKKa inactivation
k4 =    0.001;  % [0.001]    IKKii transformation


% --- IkB, A20 and TNF gene expression ------------------------------------

q1  = 4*10^-7;  % [4*10^-7] NF-kB binding at A20 and IkBa site
q2  =   10^-6;  % [10^-6]   IkBa-inducible unbinding from A20 and IkBa site
q1t = q1/10;    % [q1/10]   NF-kB binding at TNFa gene site
                %                   (q1/30 alternative to set q2tt = 0)
q2t = q2;       % [q2]      IkBa-inducible NF-kB unbinding from TNFa site
q2tt= 0.002;    % [0.002]   spontaneous NF-kB unbinding from TNFa site 
                %                   (if q2tt == 0, then set q1 = q1/30)
lambda = 0.025; % [0.025 SKNAS, 0.004 MEFs]   TNFa mRNA synthesis rate
                %                   (positive coupling)
c1 =  0.1;      % [0.1]     NF-kB-induced A20 mRNA and IkBa mRNA synthesis 
c3 =  0.00075;  % [0.00075] A20 and IkBa mRNA degradation rate,
                %                   [Blattner] t_{1/2} = 15 min  
c4  = 0.5;      % [0.5]     A20 and IkBa translation rate
c3t = 0.00075;  % [0.00075] TNFa mRNA degradation rate 
c4t = 0.05;     % [0.05]    TNFa translation rate (accounts for 3merization)



% --- Protein interactions and lifetime -----------------------------------

a1 = 5*10^-7;   % [5*10^-7] IkBA--NFkB association
a2 =   10^-7;   % [  10^-7] IkBa phosphorlyation due to action of IKKa
a3 = 5*10^-7;   % [5*10^-7] (IkBa:NFkb) phosphorylation due to action
                %                   of IKKa, at most 0.01/s
c5 = 0.0005;    % [0.0005]  A20 degradation rate
tp = 0.01;      % [0.01]    degradation of phospho-IkBa and phospho-IkBa
                %                   complexed to NF-kB
c5a = 0.0001;   % [0.0001]  IkBa degradation rate [Pando]
c5t = 0.0002;   % [0.0002]  cytoplasmic TNFa degradation  
c6a = 0.00002;  % [0.00002] spontaneous degradation of (IkBa:NFkB) to NF-kB
                %                   [Pando] 

% --- Transport -----------------------------------------------------------
 
i1  = 0.01;     % [0.01]   NFkB nuclear import 
e2a = 0.05;     % [0.05]   (IkBa:NFkB) nuclear export 
i1a = 0.002;    % [0.002]  IkBa nuclear import
e1a = 0.005;    % [0.005]  IkBa nuclear export


%% Dynamics ---------------------------------------------------------------

dy = zeros(24,1);

% active TNFR1 receptors     
dy(1) = ...
    +y(17)*csec/(y(22)+cb)*y(22) ...
    +kb*y(18)*y(22) ...
    -kf*y(1);

% active form of IKKK 
dy(2) = ...
    +ka*y(1)*y(23) * ka20/(ka20+y(7)) ...
    -ki*y(2);

% active form of IKK
dy(3) = ...
    +y(2)^2*k1*y(4) ...
    -k3*y(3) * (k2+y(7))/k2;

% neutral form of IKK
dy(4) = ...
    -y(2)^2*k1*y(4) ...
    +k4*y(24);

% inactive form of IKK
dy(5) = ...
    +k3*y(3) * (k2+y(7))/k2 ...
    -k4*y(5);

% A20 mRNA
dy(6) = ...
    +c1*y(19) ...
    -c3*y(6);

% cytoplasmic A20
dy(7) = ...
    +c4*y(6) ...
    -c5*y(7);

% IkBa mRNA
dy(8) = ...
    +c1*y(20) ...
    -c3*y(8);

% free cytoplasmic IkBa protein
dy(9) = ...
    -a2*y(3)*y(9) ...
    -a1*y(9)*y(12) ...
    +c4*y(8) ...
    -c5a*y(9) ...
    -i1a*y(9) ...
    +e1a*y(10);

% free nuclear IkBa protein
dy(10) = ...
    -a1*kv*y(10)*y(13) ...
    +i1a*y(9) ...
    -e1a*y(10);

% phospo-IkBa cytoplasmic
dy(11) = ...
    +a2*y(3)*y(9) ...
    -tp*y(11);

% free cytoplasmic NFkB
dy(12) = ...
    +c6a*y(14) ...
    -a1*y(12)*y(9) ...
    +tp*y(15) ...
    -i1*y(12);

% free nuclear NF-kB
dy(13) = ...
    +i1*y(12) ....
    -a1*kv*y(10)*y(13);

% cytoplasmic (NF-kB:IkBa) complexes
dy(14) = ...
    +a1*y(9)*y(12) ...
    -c6a*y(14) ...
    -a3*y(3)*y(14) ...
    +e2a*y(25);

% cytoplasmic (phospho-IkBa:NF-kB) complexes
dy(15) = ...
    +a3*y(3)*y(14) ...
    -tp*y(15);

% TNFa mRNA
dy(16) = ...
    +lambda*y(21) ...
    -c3t*y(16);

% intracellular TNFa
dy(17) = ...
    +c4t*y(16) ...
    -c5t*y(17) ...
    -csec*y(17);

% extracellular TNFa
dy(18) = -cdeg*y(18);

% A20 gene state
dy(19) = ...
    +q1*y(13)*(NA-y(19)) ...
    -q2*y(10)*y(19);

% IkBa gene state
dy(20) = ...
    +q1*y(13)*(NI-y(20)) ...
    -q2*y(10)*y(20);

% TNFa gene state
dy(21) = ...
    +q1t*y(13)*(NT-y(21)) ...
    -(q2tt+q2t*y(10))*y(21);

% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

% inactive TNFR1 receptors
dy(22) = ...
    -y(17)*csec/(y(22) + cb)*y(22) ...
    -kb*y(18)*y(22) ...
    +kf*y(1);

% neutral form of IKKK
dy(23) = ...
    -ka*y(1)*y(23) * ka20/(ka20+y(7)) ...
    +ki*y(2);

% inactive intermediate form of IKK
dy(24) = ...
    +k4*y(5) ...
    -k4*y(24);

% nuclear (NF-kB:IkBa) complexes
dy(25) = ...
    +a1*kv*y(10)*y(13) ...
    -e2a*y(25);

