%% The MAIN driver for numerical integration of ODEs.

% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
%                                                                         %
%         NF-kB regulatory system with autocrine TNFa stimulation         %
%                                                                         %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %

clear;              % forget all variables
clc;                % clear the comand window
starttime = clock;  % save curent time


%% TNFR1 receptors number & distribution parameters -----------------------

% Number of TNFR1 receptors:
%
% You can either set a number of receptors R directly using the median 
% value M, or use the median to draw a random number of receptors from
% a lognormal distribution with Median = M0, Mean = M0*Exp(M1^2/2) and 
% Variance = M0^2 * (Exp(M1^2) - 1) * Exp(M1^2):
%
M0 = 7000;        % median number of receptors
M1 = sqrt(0.7);
M2 = 0;
%
% a) Using median M0 directly.
%
R = M0;
%
% b) Drawing the number of receptors at random -- commented out:
%
%rand('twister', sum(1000*clock));  % initialize the random number generator
%R = round(M0*exp(M2+randn*M1));    % number of TNFR1 receptors

disp(['Number of TNFR1 receptors: ', num2str(R), '.'])


%% Alleles of NF-kB-inducible genes ---------------------------------------

NI = 2;  % number of IkBa gene alleles (formerly denoted ANa)
NA = 2;  % number of A20  gene alleles (formerly denoted AN)
NT = 2;  % number of TNFa gene alleles

%disp(['Number of IkBa gene alleles: ', num2str(NI), '.'])
disp( ['Number of A20  gene alleles: ', num2str(NA), '.'])
%disp(['Number of TNFa gene alleles: ', num2str(NT), '.'])


%% Initial conditions -----------------------------------------------------

% Initial concentration of TNFa ligands in ng/ml.
% Example doses:  [Allan]: 20, [Lee]: 10, [Hoffmann]: 1,10.
TNF = 1; 

NFkBtot = 10^5;

y0 = zeros(1,25);         % prepare a vector of initial condition, then assign:
y0(22) =    R;            % inactive TNFR1 receptors
y0(23) =   10^5;          % neutral form of IKKK (== IKKK_tot) 
y0(4)  = 2*10^5;          % neutral form IKK (== IKK_tot)
y0(14) = NFkBtot;         % cytoplasmic (NF-kB:IkBa) complexes (== NF-kB_tot)
y0(13) =    0;            % free nuclear NF-kB protein
y0(8) =    10;            % IkBa transcript
y0(9) =  0.1 *y0(14);     % free cytoplasmic IkBa protein 
y0(10) = 0.06*y0(14);     % free nuclear IkBa protein

% For A20 knock-out, initial A20 mRNA and cytoplasmic A20 are set to 0. 
if (NA > 0)
    y0(6) =   10;         % A20 mRNA
    y0(7) = 3*10^4;       % A20 protein
end

yy0=y0;                   % assign initial conditions


%% Simulation -------------------------------------------------------------

% Simulation times [seconds]:

hour= 60*60;
t00 = 100*hour;  % equilibration time
t0  =   1*hour;  % time when TNFa is added into the system
tw1 =  10*hour;  % length of TNFa stimulation
te1 =   1*hour;  % length of simulation after TNFa has been washed out

dt  = 10;        % time step (for figures) [seconds]


% Stage 0: equlibration (not shown in plots).

realtime = 0;
Y = yy0;
T = zeros(1,1); 
tspan = 0:dt:t00;
[T1,Y1] = ode23tb(@NFkB_autocrine_modelD,tspan,yy0,[],NA,NI,NT,R); 
yy0 = Y1(size(Y1,1),:);     


% Stage 1: equlibration continued (shown in plots).

if (t0 > 0)
    realtime = 0;
    Y = yy0;
    T = zeros(1,1);
    tspan = 0:dt:t0;                     
    [T0,Y0] = ode23tb(@NFkB_autocrine_modelD,tspan,yy0,[],NA,NI,NT,R); 
    T = [T;T0+realtime];
    Y = [Y;Y0];
    yy0 = Y0(size(Y0,1),:);     
end


% Stage 2: TNFa is added.

yy0(18) = TNF;  % TNFa off -> ON.

realtime = t0;
tspan = [0:dt:tw1];                   
[T0,Y0] = ode23tb(@NFkB_autocrine_modelD,tspan,yy0,[],NA,NI,NT,R); 
yy0 = Y0(size(Y0,1),:);    
T = [T;T0+realtime];
Y = [Y;Y0];


% Stage 3: TNFa is washed out.

yy0(18) = 0;  % TNFa on -> OFF.

if (te1 > 0)
    realtime = t0+tw1;
    tspan = 0:dt:te1;                   
    [T0,Y0] = ode23tb(@NFkB_autocrine_modelD,tspan,yy0,[],NA,NI,NT,R);
    T = [T;T0+realtime];
    Y = [Y;Y0];
end


%% Summary and plots ------------------------------------------------------

simulation_time = etime(clock,starttime);
%disp(['Simulation took ', num2str(simulation_time), ' s.'])

NFkB_autocrine_plots(T,Y)

